@@ -1,10 +1,12 @@
 # pylint: disable=C0103,R0903
-from typing import Union
+from typing import TypeVar, Union, cast
 
 from ansi.sequence import sequence
 
 __all__ = ['Graphic']
 
+T = TypeVar('T', str, 'Graphic')
+
 
 class Graphic(object):
     '''
@@ -15,13 +17,13 @@ class Graphic(object):
         self.values = values
         self.sequence = sequence('m', fields=-1)(*values)
 
-    def __add__(self, their: Union[str, "Graphic"]) -> Union[str, "Graphic"]:
+    def __add__(self, their: T) -> T:
         if isinstance(their, str):
-            return ''.join([str(self), their])
-        elif isinstance(their, bytes):  # type: ignore  # This is already checked by the type annotation
-            raise TypeError('You can only add strings or strings decorated with escape sequences, not bytes.')
-        else:
+            # For some reason mypy does not understand that T is string in this case
+            return cast(T, self.sequence + their)
+        elif isinstance(their, Graphic):
             return Graphic(*(self.values + their.values))
+        raise TypeError('You can only add strings or strings decorated with escape sequences together.')
 
     def __call__(self, text: str, reset: bool = True) -> str:
         result = self.sequence + text
